/*
** PUM 2008
**
** Henk Jonas
** February 2008
** metaview@web.de
**
*/

#include <PalmOS.h>
#include <hostcontrol.h>

#include <Hs.h>
//#include <HsExt.h>
//#include <HsExtCommon.h>
//#include <PhoneLib.h>
#include <VfsMgr.h>
#include <palmOneNavigator.h>

#include "nexave.h"
#include "nexave_rsc.h"
#include "utilpanl.h"

#define version50      0x04000000

static Err RomVersionCompatible (UInt32 requiredVersion, UInt16 launchFlags);
static void fastcopy(UInt16 *tmpBits1, Int32 scrWidth, Int32 scrHeight, UInt16 *tmpBits2, Int32 dstHeight);

static int  StartApplication(void);
static void StopApplication(void);

static void EventLoop(void);
static Boolean MainFormHandler(EventPtr event);
void RenderMyText(Char *string, int nLength, int x, int y, UInt8 fillRed, UInt8 fillGreen, UInt8 fillBlue, Boolean bInvert, Boolean bBorder);

#define ABS(a) (((a)<0) ? -(a) : (a))

#define	BACKGROUND_TRANSPARENT	0
#define	BACKGROUND_WHITE		1
#define	BACKGROUND_BLACK		2

typedef struct prefs
{
	Boolean bEnabled;
	Boolean bCenterButton;
	Boolean bPageDownButton;
	// Added by Shimon Shnitzer after build 1.3a: backgrounds option
	UInt8 uBackground;
	// Added by Henk Jonas
	Boolean bCatchCardInsert;
	Boolean bOptionKey;
	UInt8 u4Cast;
	UInt8 nTimeOut;
	Boolean bWipeOnClock;
} tagPrefs;
tagPrefs gblPrefs;

// struct to get the current network traffic from 2LaunchMe
struct tagExportStruct
{
	UInt32 dwVersion;
	Char name[32];
	UInt32 dwDataIn;
	UInt32 dwDataOut;
	UInt32 dwDuration;
	UInt32 dwBlockSize;
	UInt32 dwFirstSlice;
	UInt32 dwOtherSlice;
	UInt8 uInfoSelection; // since dwVersion 2: 0 - unknown, 1 - traffic, 2 - duration
};

#define NavSelectPressed(eventP)														\
(																						\
	IsFiveWayNavEvent(eventP)															\
&&  (   NavSelectPalmPressed(eventP)                                                    \
     || NavSelectHSPressed(eventP))                                                     \
)

//####################################################################################
//#### Added by Shimon Shnitzer: draw 4cast weather info line (if exists) ############
//####################################################################################

//---------------------------------------------------------------------------
// This macro return control pointer from resource ID
#define ctrlPtr(frm,id) (FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, id)))
//---------------------------------------------------------------------------

typedef struct
{
	UInt32  dwTrueForDay;

	UInt16  uWeatherIcon;
	UInt16  uAlertIcon;

	char    name[51];
	char    time[21];
	char    forecast[51];
	char    temps[51];

	// Added after build 1.90 (020): added big icon & current temp fields to info.
	char	currentTemp[51];
	UInt16	uBigWeatherIcon;
} KeyGuardInfoStruct;

void DrawPluginIcon(UInt32 dwIndex, int x, int y, Boolean bAllowHiRes);
Boolean NeedToDraw4cast();
void Update4cast(int off_y, Boolean bInvert, tagPrefs *pPrefs);

void DrawKMeteoIcon(UInt32 dwIndex, int x, int y, Boolean bAllowHiRes);
Boolean NeedToDrawKMeteo();
void UpdateKMeteo(int off_y, Boolean bInvert, tagPrefs *pPrefs);

//####################################################################################

// Added after build 6.50 (005): Treo 650 library
#ifndef sysTrapOEMDispatch
  /// OEM trap in Palm OS 3.0 and later trap table (formerly sysTrapSysReserved4)
  #define sysTrapOEMDispatch              0xA349
#endif // !sysTrapOEMDispatch
#define   sysTrapHsSelector               sysTrapOEMDispatch
#define hsSelGetPhoneLibrary              0x86
extern Err HsGetPhoneLibrary (UInt16 * refNum)
            SYS_SEL_TRAP (sysTrapHsSelector, hsSelGetPhoneLibrary);

/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *                             
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	11/15/96	Initial Revision
 *			hjo	02/07/02	
 *
 ***********************************************************************/
static
Err RomVersionCompatible (UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
	{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
		{
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < 0x02000000)
			{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		}
		
		return (sysErrRomIncompatible);
	}

	return (0);
}


static
int maximum(int a, int b)
{
	if (a > b)
		return a;

	return b;
}


#define TREO600 1
#define TREO650 2
#define TREO700 3
#define TREO680 4
#define TREO755 5
#define CENTRO_GSM  6
#define CENTRO_CDMA 7
#define TREOANY 99

static
int CheckForTreo()
{
	// check for Treo 600 to use HS extensions
	UInt32 deviceID, companyID;
	Err devErr, compErr;
	int result = 0;

	devErr = FtrGet(sysFtrCreator, sysFtrNumOEMDeviceID, &deviceID);
	compErr = FtrGet(sysFtrCreator, sysFtrNumOEMCompanyID, &companyID);
	if ((devErr == errNone) && (compErr == errNone))
	{
		if ((deviceID == 'H201') || (deviceID == 'H101'))
			result = TREO600;
		else if ((deviceID == 'H202') || (deviceID == 'H102'))
			result = TREO650;
		else if (deviceID == 'D052')
			result = TREO700;
		else if (deviceID == 'D053')
			result = TREO680;
		else if (deviceID == 'D060')
			result = TREO755;
		else if (deviceID == 'D061')
			result = CENTRO_GSM;
		else if (deviceID == 'D062')
			result = CENTRO_CDMA;
		else
		{
			UInt32 version = 0;
			Err err = FtrGet (hsFtrCreator, hsFtrIDVersion, &version);
			if (err == errNone)
				result = TREOANY;
		}
	}

	return result;
}

static
void ReadPrefs(tagPrefs *pPrefs)
{
	UInt16 size = sizeof(tagPrefs);
	MemSet(pPrefs, size, 0);

	// Added by Shimon Shnitzer after build 1.3a: default unlocking methods to ON
	pPrefs->bCenterButton    = true;
	pPrefs->bPageDownButton  = true;
	pPrefs->bCatchCardInsert = true;
	pPrefs->u4Cast = 2;
	pPrefs->nTimeOut = 5;

	PrefGetAppPreferences(MyAppID, 1, pPrefs, &size, false);

	if (pPrefs->nTimeOut > 20)
		pPrefs->nTimeOut = 20;
	else if (pPrefs->nTimeOut < 1)
		pPrefs->nTimeOut = 1;
}

static
void RegisterNotifications()
{
	UInt16 cardNo;
	LocalID dbID;

	if (SysCurAppDatabase( &cardNo, &dbID ) == errNone)
	{
		if (CheckForTreo())
			SysNotifyRegister( cardNo, dbID, sysNotifyVirtualCharHandlingEvent, NULL, 32, NULL );
		else
			SysNotifyRegister( cardNo, dbID, sysNotifyLateWakeupEvent, NULL, 0, NULL );
		SysNotifyRegister( cardNo, dbID, sysNotifyVolumeMountedEvent, NULL, -32, NULL );
		SysNotifyRegister( cardNo, dbID, pmNotifyHighPriorityUITask, NULL, sysNotifyNormalPriority, NULL);
	}
}


static
void UnregisterNotifications()
{
	UInt16 cardNo;
	LocalID dbID;

	if (SysCurAppDatabase( &cardNo, &dbID ) == errNone)
	{
		if (CheckForTreo())
			SysNotifyUnregister( cardNo, dbID, sysNotifyVirtualCharHandlingEvent, 32);
		else
			SysNotifyUnregister( cardNo, dbID, sysNotifyLateWakeupEvent, 0);
		SysNotifyUnregister( cardNo, dbID, sysNotifyVolumeMountedEvent, -32 );
	}
}


static
Boolean MainFormHandler(EventPtr event)
{
	FormPtr   form;
	Boolean   handled = false;
	UInt16    sel;

	form = FrmGetActiveForm();

	switch (event->eType) 
	{
		case frmOpenEvent:
			{
				UInt16 value;

				// Initialize pref panel
				PanelFormInit(FrmGetActiveForm());
				FrmSetControlValue(form, FrmGetObjectIndex(form, EnableButton), gblPrefs.bEnabled);
				FrmSetControlValue(form, FrmGetObjectIndex(form, DisableButton), !gblPrefs.bEnabled);
				FrmSetControlValue(form, FrmGetObjectIndex(form, CenterButtonCheck), gblPrefs.bCenterButton);
				FrmSetControlValue(form, FrmGetObjectIndex(form, PageDownButtonCheck), gblPrefs.bPageDownButton);
				FrmSetControlValue(form, FrmGetObjectIndex(form, CaptureCardCheck), gblPrefs.bCatchCardInsert);
				FrmSetControlValue(form, FrmGetObjectIndex(form, OptionButtonCheck), gblPrefs.bOptionKey);
				FrmSetControlValue(form, FrmGetObjectIndex(form, OnClockButtonCheck), gblPrefs.bWipeOnClock);
				value = gblPrefs.nTimeOut;
				CtlSetSliderValues(FrmGetObjectPtr(form, FrmGetObjectIndex(form, TimeOutSlider)), NULL, NULL, NULL, &value);

				// Added by Shimon Shnitzer after build 1.3a: backgrounds option
				LstSetSelection(ctrlPtr(form, lstBackground), gblPrefs.uBackground);
				CtlSetLabel(ctrlPtr(form, triggerBackground), LstGetSelectionText(ctrlPtr(form,lstBackground), gblPrefs.uBackground));

				// Added by Henk Jonas after build 1.3f: 4cast option
				LstSetSelection(ctrlPtr(form, lst4Cast), gblPrefs.u4Cast);
				CtlSetLabel(ctrlPtr(form, trigger4Cast), LstGetSelectionText(ctrlPtr(form, lst4Cast), gblPrefs.u4Cast));

//				if (gblPrefs.bEnabled)
				{
					RegisterNotifications();
				}
				
				FrmAlert(AboutAlert);

				FrmDrawForm(form);
				handled = true;
			}
			break;
		
		case ctlSelectEvent:  // A control button was pressed and released.
			// Close button
			handled = PanelSelectClose(event);
			if (!handled)
			{
				switch (event->data.ctlSelect.controlID)
				{
					// Added by Shimon Shnitzer after build 1.3a: backgrounds option
					case triggerBackground:
						sel = LstPopupList(ctrlPtr(form,lstBackground));
						if (sel!=noListSelection)
						{
							gblPrefs.uBackground = sel;
							CtlSetLabel(ctrlPtr(form,triggerBackground), LstGetSelectionText(ctrlPtr(form,lstBackground),gblPrefs.uBackground));
						
							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(tagPrefs), false);
							handled = true;
						}
						break;

					case trigger4Cast:
						sel = LstPopupList(ctrlPtr(form,lst4Cast));
						if (sel!=noListSelection)
						{
							gblPrefs.u4Cast = sel;
							CtlSetLabel(ctrlPtr(form,trigger4Cast), LstGetSelectionText(ctrlPtr(form,lst4Cast),gblPrefs.u4Cast));
						
							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(tagPrefs), false);
							handled = true;
						}
						break;

					case EnableButton:
						{
							gblPrefs.bEnabled = true;
							RegisterNotifications();

							// Added by Shimon Shnitzer after build 1.3a: 'Enable' & 'Disable' buttons are checkboxes
							FrmSetControlValue(form, FrmGetObjectIndex(form, EnableButton) , gblPrefs.bEnabled);
							FrmSetControlValue(form, FrmGetObjectIndex(form, DisableButton), !gblPrefs.bEnabled);

							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(tagPrefs), false);
							handled = true;
						}
						break;
	
					case DisableButton:
						{
							gblPrefs.bEnabled = false;
//							UnregisterNotifications();

							// Added by Shimon Shnitzer after build 1.3a: 'Enable' & 'Disable' buttons are checkboxes
							FrmSetControlValue(form, FrmGetObjectIndex(form, EnableButton) , gblPrefs.bEnabled);
							FrmSetControlValue(form, FrmGetObjectIndex(form, DisableButton), !gblPrefs.bEnabled);

							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(tagPrefs), false);
							handled = true;
						}
						break;
	
					case CenterButtonCheck:
						{
							gblPrefs.bCenterButton = FrmGetControlValue(form, FrmGetObjectIndex(form, CenterButtonCheck));
							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(gblPrefs), false);
							handled = true;
						}
						break;
	
					case PageDownButtonCheck:
						{
							gblPrefs.bPageDownButton = FrmGetControlValue(form, FrmGetObjectIndex(form, PageDownButtonCheck));
							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(gblPrefs), false);
							handled = true;
						}
						break;
	
					case OptionButtonCheck:
						{
							gblPrefs.bOptionKey = FrmGetControlValue(form, FrmGetObjectIndex(form, OptionButtonCheck));
							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(gblPrefs), false);
							handled = true;
						}
						break;

					case OnClockButtonCheck:
						{
							gblPrefs.bWipeOnClock = FrmGetControlValue(form, FrmGetObjectIndex(form, OnClockButtonCheck));
							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(gblPrefs), false);
							handled = true;
						}
						break;

					case CaptureCardCheck:
						{
							gblPrefs.bCatchCardInsert = FrmGetControlValue(form, FrmGetObjectIndex(form, CaptureCardCheck));
							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(gblPrefs), false);
							handled = true;
						}
						break;

					case TimeOutSlider:
						{
							UInt16 value;
							CtlGetSliderValues(FrmGetObjectPtr(form, FrmGetObjectIndex(form, TimeOutSlider)), NULL, NULL, NULL, &value);
							gblPrefs.nTimeOut = value;
							PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(gblPrefs), false);
							handled = true;
						}
						break;
				}
			}
			break;

		case ctlRepeatEvent:
			{
				RectangleType rect;
				UInt16 min, max;
				Char strValue[16];

				CtlGetSliderValues(ctrlPtr(form, event->data.ctlRepeat.controlID), &min, &max, NULL, NULL);
				FrmGetObjectBounds(form, FrmGetObjectIndex(form, event->data.ctlRepeat.controlID), &rect);
				StrPrintF(strValue, "%d", event->data.ctlRepeat.value);

				rect.topLeft.x -= FntCharsWidth("xxx", 3);
				rect.extent.x = FntCharsWidth("xxx", 3);
				WinPushDrawState();
				WinSetBackColor(12);
				WinSetForeColor(12);
				WinSetTextColor(255);
				WinDrawRectangle(&rect, 3);
				WinDrawChars(strValue, StrLen(strValue), rect.topLeft.x + (rect.extent.x - FntCharsWidth(strValue, StrLen(strValue))) / 2, rect.topLeft.y+2);
				WinPopDrawState();
			}
			break;

		case menuEvent:
			switch (event->data.menu.itemID)
			{
				case HelpItem:
					FrmHelp(HelpString);
					break;
				case AboutItem:
					FrmAlert(AboutAlert);
					break;
				default:
					break;
			}
			handled = true;
			break;

		case popSelectEvent:
			handled = PanelSelectPanel(event);
			break;

		case frmCloseEvent:
			PanelFormClose();
			break;

		default:
			break;
	}
	return handled;
}


static
void EventLoop(void)
{
	Err err;
	EventType event;

	do
	{
		EvtGetEvent(&event, evtWaitForever);

		if (SysHandleEvent(&event))
			continue;
		if (MenuHandleEvent((void *)0, &event, &err))
			continue;

		if (event.eType == frmLoadEvent)
		{
			UInt16 formID = event.data.frmLoad.formID;
			FormPtr form = FrmInitForm(formID);
			FrmSetActiveForm(form);
			switch (formID) 
			{
				case MainForm:
					FrmSetEventHandler(form, (FormEventHandlerPtr) MainFormHandler);
					break;
				default:
					break;
			}
		}
		FrmDispatchEvent(&event);
	} while(event.eType != appStopEvent);
}


static
int StartApplication(void)
{
	ReadPrefs(&gblPrefs);

	FrmGotoForm(MainForm);
	
	return 0;
}


static
void StopApplication(void)
{
	/* Close all the open forms. */
	FrmCloseAllForms();

	PrefSetAppPreferences(MyAppID, 1 /* ID */, 1 /* Version */, &gblPrefs, sizeof(gblPrefs), false);
}


// black border, white fill
void RenderMyText(Char *string, int nLength, int x, int y, UInt8 fillRed, UInt8 fillGreen, UInt8 fillBlue, Boolean bInvert, Boolean bBorder)
{
	RGBColorType rgb;
	UInt32 locRomVersion;

	WinPushDrawState();
	WinSetDrawMode(winOverlay);
	
	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &locRomVersion);
	if (locRomVersion >= 0x05000000)
	{
		Boolean locHighDensityFeature = false;
		UInt32 screenAttr = kDensityLow;
		UInt32 version = 0;

		if (FtrGet(sysFtrCreator, sysFtrNumWinVersion, &version) == errNone)
		{
			if (version >= 4)
				locHighDensityFeature = true;
		}

		if (x == 0)
			x = 1;
		if (y == 0)
			y = 1;

		if (locHighDensityFeature)
			WinScreenGetAttribute(winScreenDensity, &screenAttr);
	
		if (screenAttr != kDensityLow)
		{
			WinSetCoordinateSystem(kCoordinatesDouble);
			y <<= 1;
			x <<= 1;
		}

		if (bBorder)
		{
			if (bInvert)
			{
				rgb.r = fillRed;
				rgb.g = fillGreen;
				rgb.b = fillBlue;
			}
			else
			{
				rgb.r = 0;
				rgb.g = 0;
				rgb.b = 0;
			}
			WinSetTextColorRGB(&rgb, NULL);
			WinPaintChars(string, nLength, x-1, y);
			WinPaintChars(string, nLength, x+1, y);
			WinPaintChars(string, nLength, x, y-1);
			WinPaintChars(string, nLength, x, y+1);
		}

		if (bInvert)
		{
			rgb.r = 0;
			rgb.g = 0;
			rgb.b = 0;
		}
		else
		{
			rgb.r = fillRed;
			rgb.g = fillGreen;
			rgb.b = fillBlue;
		}
		WinSetTextColorRGB(&rgb, NULL);
	}
	else
	{
		rgb.r = 0;
		rgb.g = 0;
		rgb.b = 0;
		WinSetTextColorRGB(&rgb, NULL);
	}

	WinPaintChars(string, nLength, x, y);
	WinPopDrawState();
}


//####################################################################################
//#### Added by Shimon Shnitzer: draw 4cast weather info line (if exists) ############
//####################################################################################

void DrawPluginIcon(UInt32 dwIndex, int x, int y, Boolean bAllowHiRes)
{
    DmOpenRef dbP;
    Boolean   bRes = false;
    UInt16    wResourceType;
    UInt32    dwSize;
    MemHandle recordH;
    BitmapPtr record;
    int       rx,ry;

    wResourceType = -1;
    // First try overlays:
    dbP = DmOpenDatabaseByTypeCreator('appl','cas4',dmModeReadOnly);
    if (dbP!=NULL)
    {
        wResourceType = DmFindResource(dbP,'Tbmp',dwIndex,NULL);

        if (wResourceType!=-1)
        {
            recordH = DmGetResourceIndex (dbP,wResourceType);

            if (recordH!=NULL)
            {
                dwSize = MemHandleSize(recordH);
                record = MemHandleLock (recordH);
                if (record!=NULL)
                {
                	rx = x;
                    ry = y;
                    //if ((!bAllowHiRes) || (!IsHiResMode()))
                    //{
                        WinDrawBitmap(record,rx,ry); 
                    //} else {
                    //    DrawHRResource(record,rx,ry,false);
                    //}

                    MemHandleUnlock(recordH);
                    bRes = true;
                }

                DmReleaseResource(recordH);
            }
        }

        DmCloseDatabase(dbP);
    }
}

//-----------------------------------------------------------------------------------

Boolean NeedToDraw4cast()
{
	DmSearchStateType sst;
	UInt16            cardNo;
	LocalID           dbID;
	
	return (DmGetNextDatabaseByTypeCreator(true,&sst,'kgdt','cas4',true,&cardNo,&dbID)==errNone);
}

//-----------------------------------------------------------------------------------

void Update4cast(int y, Boolean bInvert, tagPrefs *pPrefs)
{
	DmOpenRef          dbP;
	MemHandle          hRec;
	KeyGuardInfoStruct *pRec;
	Int16              off_y,off_x,sel,dx,len;
	UInt32             dwNow;
	char               txt[101];
	Boolean            bFit;
	RGBColorType rgbWhite = {0, 255, 255, 255};
	RGBColorType rgbBlack = {0,   0,   0,   0};
	DateTimeType dt;
	
	dbP = DmOpenDatabaseByTypeCreator('kgdt','cas4',dmModeReadOnly);	
	if (dbP==NULL) return;
	
	dwNow = TimGetSeconds()/86400;
	TimSecondsToDateTime(TimGetSeconds(), &dt);
	// added by Henk Jonas after 1.3f
	if ((pPrefs->u4Cast == 2) && (dt.hour >= 12))
		dwNow += 1;
	else if (pPrefs->u4Cast == 3)
		dwNow += 1;
	TimSecondsToDateTime(dwNow * 86400, &dt);
	
	off_y = y+5;
	if (bInvert)
	{
		WinSetForeColorRGB(&rgbBlack, NULL);
	} else {
		WinSetForeColorRGB(&rgbWhite, NULL);
	}
	WinDrawLine(2,y,157,y);
	
	hRec = DmQueryRecord(dbP,0);
	if (hRec!=NULL)
	{
		pRec = MemHandleLock(hRec);
		if (pRec!=NULL)
		{
			sel   = 0;
			off_x = 0;
			
			// Check which record fits today (if any...)
			for (sel=0;sel<5;sel++)
			{
				if (pRec[sel].dwTrueForDay==dwNow) break;
			}
			if (sel==5)
			{
				StrCopy(txt,"No forecast...");
				FntSetFont(stdFont);
				RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
			}
			else if (pPrefs->u4Cast < 4)
			{
				FntSetFont(stdFont);
				DateTemplateToAscii("^1s:", dt.month, dt.day, dt.year, txt, sizeof(txt));
				RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
				off_x += FntCharsWidth(txt, StrLen(txt))+1;

				// Weather icon:
				if (pRec[sel].uWeatherIcon!=0)
				{
					DrawPluginIcon(pRec[sel].uWeatherIcon,off_x,off_y,true);
					off_x+=14;
				}
				
				// Alert icon (if exists):
				if (pRec[sel].uAlertIcon!=0)
				{
					DrawPluginIcon(pRec[sel].uAlertIcon,off_x,off_y,true);
					off_x+=12;
				}
				
				// City name:
				FntSetFont(stdFont);
				StrPrintF(txt,"(%s)",pRec[sel].name);
				RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
				off_x += FntCharsWidth(txt, StrLen(txt))+1;
				
				// time (if exists):
				if (pRec[sel].time[0]!='\0')
				{
					FntSetFont(stdFont);				
					StrCopy(txt, pRec[sel].time);
					RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
					off_x += FntCharsWidth(txt, StrLen(txt))+1;
				}

				// Temps:
				FntSetFont(stdFont);
				dx = FntCharsWidth(pRec[sel].temps, StrLen(pRec[sel].temps));
				RenderMyText(pRec[sel].temps, StrLen(pRec[sel].temps), 159-dx, off_y, 255, 255, 255, bInvert, !bInvert);
				//off_x += FntCharsWidth(pRec[sel].temps, StrLen(pRec[sel].temps));
				
				// Forecast:
				FntSetFont(boldFont);
				StrCopy(txt, pRec[sel].forecast);
				dx = 159-dx-off_x-1;
				len = StrLen(txt);				
				FntCharsInWidth(txt, &dx, &len, &bFit);				
				RenderMyText(txt, len, off_x, off_y, 255, 255, 255, bInvert, !bInvert);
			}
			else
			{
				int i;
				for (i=0; i<3; i++)
				{
					off_x = i * 53;

					FntSetFont(stdFont);
					TimSecondsToDateTime((dwNow + i) * 86400, &dt);
					DateTemplateToAscii("^1s:", dt.month, dt.day, dt.year, txt, sizeof(txt));
					RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
					off_x += FntCharsWidth(txt, StrLen(txt))+1;

					if ((sel+i) < 5)
					{
						// Weather icon:
						if (pRec[sel+i].uWeatherIcon!=0)
						{
							DrawPluginIcon(pRec[sel+i].uWeatherIcon,off_x,off_y,true);
						}
						off_x+=14;

						// Temps:
						StrCopy(txt, pRec[sel+i].temps);
						FntSetFont(stdFont);
						dx = (i + 1) * 53 - off_x;
						len = StrLen(txt);				
						FntCharsInWidth(txt, &dx, &len, &bFit);				
						RenderMyText(txt, len, off_x, off_y, 255, 255, 255, bInvert, !bInvert);
					}
					else
					{
						StrCopy(txt,"xxx");
						FntSetFont(stdFont);
						dx = (i + 1) * 53 - off_x;
						len = StrLen(txt);				
						FntCharsInWidth(txt, &dx, &len, &bFit);				
						RenderMyText(txt, len, off_x, off_y, 255, 255, 255, bInvert, !bInvert);
					}
				}
			}

			MemHandleUnlock(hRec);
		}
	}		
	DmCloseDatabase(dbP);
}


//####################################################################################
//#### Added by Henk Jonas: draw kMeteo weather info line (if exists) ############
//####################################################################################

void DrawKMeteoIcon(UInt32 dwIndex, int x, int y, Boolean bAllowHiRes)
{
	DmOpenRef dbP;
	UInt16    wResourceType;
	MemHandle recordH;
	BitmapPtr record;

	wResourceType = -1;
	// First try overlays:
	dbP = DmOpenDatabaseByTypeCreator('icon', 'kMto', dmModeReadOnly);
	if (dbP != NULL)
	{
		wResourceType = DmFindResource(dbP, 'Tbmp', dwIndex, NULL);

		if (wResourceType!=-1)
		{
			recordH = DmGetResourceIndex (dbP, wResourceType);

			if (recordH)
			{
				record = MemHandleLock(recordH);
				if (record)
				{
					WinPaintBitmap(record, x, y);
					MemHandleUnlock(recordH);
				}

				DmReleaseResource(recordH);
			}
		}

		DmCloseDatabase(dbP);
	}
}

//-----------------------------------------------------------------------------------

Boolean NeedToDrawKMeteo()
{
	DmSearchStateType sst;
	UInt16            cardNo;
	LocalID           dbID;
	
	return (DmGetNextDatabaseByTypeCreator(true, &sst, 'DXML','kMto', true, &cardNo, &dbID)==errNone);
}

//-----------------------------------------------------------------------------------

void UpdateKMeteo(int y, Boolean bInvert, tagPrefs *pPrefs)
{
	DmOpenRef          dbP;
	MemHandle          hRec;
	Char              *pRec;
	Int16              off_y,off_x,sel,dx,len,i;
	UInt32             dwNow;
	char               txt[101];
	Boolean            bFit;
	RGBColorType rgbWhite = {0, 255, 255, 255};
	RGBColorType rgbBlack = {0,   0,   0,   0};
	DateTimeType dt;
	
	dbP = DmOpenDatabaseByTypeCreator('DXML','kMto',dmModeReadOnly);	
	if (dbP==NULL) return;
	
	dwNow = TimGetSeconds()/86400;
	TimSecondsToDateTime(TimGetSeconds(), &dt);
	// added by Henk Jonas after 1.3f
	if ((pPrefs->u4Cast == 2) && (dt.hour >= 12))
		dwNow += 1;
	else if (pPrefs->u4Cast == 3)
		dwNow += 1;
	TimSecondsToDateTime(dwNow * 86400, &dt);
	
	off_y = y+5;
	if (bInvert)
	{
		WinSetForeColorRGB(&rgbBlack, NULL);
	} else {
		WinSetForeColorRGB(&rgbWhite, NULL);
	}
	WinDrawLine(2,y,157,y);
	
	hRec = DmQueryRecord(dbP,0);
	if (hRec!=NULL)
	{
		pRec = MemHandleLock(hRec);
		if (pRec!=NULL)
		{
			Char *ptr;

			sel   = 0;
			off_x = 0;
			
			ptr = StrStr(pRec, "<dayf>");
			if (ptr)
				ptr = StrStr(ptr, "<lsup>");
			if (ptr)
			{
				DateType date;
				UInt32 dwLastUpdate;
				Int16 nHighValue = -9999;
				Int16 nLowValue = -9999;
				Int16 nIcon = 0;

				ptr += 6;
				date.month = StrAToI(ptr);
				ptr += 2;
				if (date.month > 9)
					ptr++;
				date.day = StrAToI(ptr);
				ptr += 2;
				if (date.day > 9)
					ptr++;
				date.year = StrAToI(ptr) + 2000 - 1904;
				dwLastUpdate = DateToDays(date);
				
				if (dwLastUpdate <= (dwNow - 10))
				{
					StrCopy(txt,"No forecast...");
					FntSetFont(stdFont);
					RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
				}
				else if (pPrefs->u4Cast < 4)
				{
					FntSetFont(stdFont);
					DateTemplateToAscii("^1s:", dt.month, dt.day, dt.year, txt, sizeof(txt));
					RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
					off_x += FntCharsWidth(txt, StrLen(txt))+1;

					// Weather icon:
					ptr = StrStr(pRec, "<dayf>");
					if (ptr)
						ptr += 6;
					for (i=0; i<=(dwNow-dwLastUpdate); i++)
					{
						if (ptr)
							ptr = StrStr(ptr, "<day ");
						if (ptr)
							ptr += 5;
					}
					if (ptr)
						ptr = StrStr(ptr, "<icon>");
					if (ptr)
					{
						ptr += 6;
						nIcon = StrAToI(ptr);
					}
					if (nIcon != 0)
					{
						DrawKMeteoIcon(nIcon, off_x, off_y, true);
						off_x+=14;
					}

					// City name:
					ptr = StrStr(pRec, "<loc ");
					if (ptr)
						ptr = StrStr(ptr, "<dnam>");
					if (ptr)
					{
						ptr += 6;
						FntSetFont(stdFont);
						StrCopy(txt, "(");
						StrNCat(txt, ptr, 7);
						txt[7] = 0;
						StrCat(txt, ")");
						RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
						off_x += FntCharsWidth(txt, StrLen(txt))+1;
					}

					// Temps:
					ptr = StrStr(pRec, "<dayf>");
					if (ptr)
						ptr += 6;
					for (i=0; i<=(dwNow-dwLastUpdate); i++)
					{
						if (ptr)
							ptr = StrStr(ptr, "<day ");
						if (ptr)
							ptr += 5;
					}
					if (ptr)
						ptr = StrStr(ptr, "<hi>");
					if (ptr)
					{
						ptr += 4;
						nHighValue = StrAToI(ptr);
					}

					ptr = StrStr(pRec, "<dayf>");
					if (ptr)
						ptr += 6;
					for (i=0; i<=(dwNow-dwLastUpdate); i++)
					{
						if (ptr)
							ptr = StrStr(ptr, "<day ");
						if (ptr)
							ptr += 5;
					}
					if (ptr)
						ptr = StrStr(ptr, "<low>");
					if (ptr)
					{
						ptr += 5;
						nLowValue = StrAToI(ptr);
					}
					if ((nHighValue != -9999) && (nLowValue != -9999))
					{
						StrPrintF(txt, "%d/%d", nHighValue, nLowValue);
						FntSetFont(stdFont);
						dx = FntCharsWidth(txt, StrLen(txt));
						RenderMyText(txt, StrLen(txt), 159-dx, off_y, 255, 255, 255, bInvert, !bInvert);
					}
					
					// Forecast:
					ptr = StrStr(pRec, "<dayf>");
					if (ptr)
						ptr += 6;
					for (i=0; i<=(dwNow-dwLastUpdate); i++)
					{
						if (ptr)
							ptr = StrStr(ptr, "<day ");
						if (ptr)
							ptr += 5;
					}
					if (ptr)
						ptr = StrStr(ptr, "<bt>");
					if (ptr)
					{
						Char *ptr2;

						ptr += 4;
						ptr2 = StrStr(ptr, "</bt>");
						if (ptr2)
						{
							FntSetFont(boldFont);
							StrNCopy(txt, ptr, ptr2-ptr);
							txt[ptr2-ptr] = 0;
							dx = 159-dx-off_x-1;
							len = StrLen(txt);				
							FntCharsInWidth(txt, &dx, &len, &bFit);				
							RenderMyText(txt, len, off_x, off_y, 255, 255, 255, bInvert, !bInvert);
						}
					}
				}
				else
				{
					int day;
					for (day=0; day<3; day++)
					{
						off_x = day * 53;

						FntSetFont(stdFont);
						TimSecondsToDateTime((dwNow + day) * 86400, &dt);
						DateTemplateToAscii("^1s:", dt.month, dt.day, dt.year, txt, sizeof(txt));
						RenderMyText(txt, StrLen(txt), off_x, off_y, 255, 255, 255, bInvert, !bInvert);
						off_x += FntCharsWidth(txt, StrLen(txt))+1;

						if (dwLastUpdate <= (dwNow+day - 10))//(((dwNow-dwLastUpdate)+day) < 10)
						{
							StrCopy(txt,"xxx");
							FntSetFont(stdFont);
							dx = (day + 1) * 53 - off_x;
							len = StrLen(txt);				
							FntCharsInWidth(txt, &dx, &len, &bFit);				
							RenderMyText(txt, len, off_x, off_y, 255, 255, 255, bInvert, !bInvert);
						}
						else
						{
							// Weather icon:
							ptr = StrStr(pRec, "<dayf>");
							if (ptr)
								ptr += 6;
							for (i=0; i<=((dwNow-dwLastUpdate)+day); i++)
							{
								if (ptr)
									ptr = StrStr(ptr, "<day ");
								if (ptr)
									ptr += 5;
							}
							if (ptr)
								ptr = StrStr(ptr, "<icon>");
							if (ptr)
							{
								ptr += 6;
								nIcon = StrAToI(ptr);
							}
							if (nIcon != 0)
							{
								DrawKMeteoIcon(nIcon, off_x, off_y, true);
								off_x+=14;
							}

							ptr = StrStr(pRec, "<dayf>");
							if (ptr)
								ptr += 6;
							for (i=0; i<=((dwNow-dwLastUpdate)+day); i++)
							{
								if (ptr)
									ptr = StrStr(ptr, "<day ");
								if (ptr)
									ptr += 5;
							}
							if (ptr)
								ptr = StrStr(ptr, "<hi>");
							if (ptr)
							{
								ptr += 4;
								nHighValue = StrAToI(ptr);
							}

							ptr = StrStr(pRec, "<dayf>");
							if (ptr)
								ptr += 6;
							for (i=0; i<=((dwNow-dwLastUpdate)+day); i++)
							{
								if (ptr)
									ptr = StrStr(ptr, "<day ");
								if (ptr)
									ptr += 5;
							}
							if (ptr)
								ptr = StrStr(ptr, "<low>");
							if (ptr)
							{
								ptr += 5;
								nLowValue = StrAToI(ptr);
							}
							if ((nHighValue != -9999) && (nLowValue != -9999))
							{
								StrPrintF(txt, "%d/%d", nHighValue, nLowValue);
								FntSetFont(stdFont);
								dx = (day + 1) * 53 - off_x;
								len = StrLen(txt);				
								FntCharsInWidth(txt, &dx, &len, &bFit);				
								RenderMyText(txt, len, off_x, off_y, 255, 255, 255, bInvert, !bInvert);
							}
						}
					}
				}
			}

			MemHandleUnlock(hRec);
		}
	}		
	DmCloseDatabase(dbP);
}

//####################################################################################



/***********************************************************************
 *
 * FUNCTION:    FirstDayOfYear
 *
 * DESCRIPTION: Return the number of day from 1/1/1904 of the first day 
 *              of the year passed.
 *
 *					 The first day of the year is always a Monday.  The rule 
 *					 for determining the first day of the year is:
 *				
 *					 New Years Day	First Day of the Year
 *					 ------------	---------------------
 *					 Monday			Monday Jan 1
 *					 Tuesday			Monday Dec 31
 *					 Wednesday		Monday Dec 30
 *					 Thursday		Monday Dec 29
 *					 Friday			Monday Jan 4 
 *					 Saturday		Monday Jan 3 
 *					 Sunday			Monday Jan 2
 *
 * PARAMETERS:	 year  - year (1904-2031)
 *
 * RETURNED:	 number of days since 1/1/1904
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/4/96	Initial Revision
 *
 ***********************************************************************/
static
UInt32 FirstDayOfYear (UInt16 year)
{
	UInt32 days;
	UInt16 dayOfWeek;
	DateType date;
	
	// Get days to January 1st of the year passed.
	date.day = 1;
	date.month = 1;
	date.year = year - firstYear;
	days = DateToDays (date);
	
	dayOfWeek = DayOfWeek (1, 1, year);

	// Move to monday.
	days++;
	days -= dayOfWeek;

	
	if (dayOfWeek >= friday)
		days += daysInWeek;
		
	return (days);
}


/***********************************************************************
 *
 * FUNCTION:    GetWeekNumber
 *
 * DESCRIPTION: Calculate the week number of the specified date.
 *
 * PARAMETERS:	 month - month (1-12)
 *              day   - day (1-31)
 *              year  - year (1904-2031)
 *
 * RETURNED:	 week number (1-53)
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/4/96	Initial Revision
 *
 ***********************************************************************/
static
UInt16 GetWeekNumber (UInt16 month, UInt16 day, UInt16 year)
{
	UInt16 dow;
	UInt16 week;
	UInt32 days;
	UInt32 firstOfYear;
	UInt32 firstOfNextYear;
	DateType date;

	// Calculate the julian date of Monday in the same week as the 
	// specified date.
	date.day = day;
	date.month = month;
	date.year = year - firstYear;
	days = DateToDays (date);

	// Adjust the day of the week by the preference setting for the first day
	// of the week.
	dow = (DayOfWeek(month, day, year) - PrefGetPreference(prefWeekStartDay) + daysInWeek) % daysInWeek;

	if (monday < PrefGetPreference(prefWeekStartDay))
		days -= (Int16)dow - (monday + daysInWeek - PrefGetPreference(prefWeekStartDay));
	else		
		days -= (Int16)dow - (monday - (Int16)PrefGetPreference(prefWeekStartDay));


	firstOfYear = FirstDayOfYear (year);
	
	if (days < firstOfYear)
	{
		// The date passed is in a week that is part of the prior 
		//	year, so get the start of the prior year.
		if (year > firstYear)
			firstOfYear = FirstDayOfYear (--year);
	}
	else
	{
		// Make sure the date passed is not in a week that in part
		// of next year.
		if (year < lastYear)
		{  
			firstOfNextYear = FirstDayOfYear (year + 1);
			if (days == firstOfNextYear)
				firstOfYear = firstOfNextYear;
		}
	}
	
	week = ((Int16)(days - firstOfYear)) / daysInWeek + 1;		// one base
	
	return (week);
}


static
void DoTheUpdate(char *ptrPhoneState, Int16 scrWidth, Int16 scrHeight, Int16 dstHeight, Int16 nTop, WinHandle tmpWnd1, WinHandle tmpWnd2, WinHandle tmpWnd3, WinHandle scrWnd, tagPrefs *pPrefs)
{
	RectangleType scrRect;
	Boolean       bWhite = (pPrefs->uBackground==1);
	RGBColorType rgbRed = {0, 255, 0, 0};
	RGBColorType rgbYellow = {0, 255, 255, 0};
	RGBColorType rgbGreen = {0, 0, 255, 0};
	RGBColorType rgbBlack = {0, 0, 0, 0};
	RGBColorType rgbWhite = {0, 255, 255, 255};
	UInt32 attr;

	WinPushDrawState();
	WinSetDrawWindow(tmpWnd3);

	WinScreenGetAttribute(winScreenDensity, &attr);
	if (attr == kDensityDouble)
		WinSetCoordinateSystem(kCoordinatesDouble);

	// prepare our offscreen window (tmpWnd3)
	// 1. draw the screen (tmpWnd1)
	// do the magic:
	// 2. draw the transparent bitmap (tmpWnd2)
	// 3. draw the gadgets
	// 4. display our offscreen window (tmpWnd3)

	// 1. draw our screen image
	scrRect.topLeft.x = 0;
	scrRect.topLeft.y = 0;
	scrRect.extent.x = scrWidth;
	scrRect.extent.y = scrHeight;
	WinCopyRectangle(tmpWnd1, tmpWnd3, &scrRect, 0, 0, winPaint);

	// 2. draw the transparent bitmap
	scrRect.topLeft.x = 0;
	scrRect.topLeft.y = nTop - (scrHeight - dstHeight) + 6;
	scrRect.extent.x = scrWidth;
	scrRect.extent.y = dstHeight;
	WinCopyRectangle(tmpWnd2, tmpWnd3, &scrRect, 0, nTop + 6, winPaint);

	// with round corners???
	scrRect.topLeft.x = 6;
	scrRect.topLeft.y = nTop - (scrHeight - dstHeight);
	scrRect.extent.x = scrWidth - 12;
	scrRect.extent.y = 1;
	WinCopyRectangle(tmpWnd2, tmpWnd3, &scrRect, 6, nTop, winPaint);
	scrRect.topLeft.x = 4;
	scrRect.topLeft.y = nTop - (scrHeight - dstHeight) + 1;
	scrRect.extent.x = scrWidth - 8;
	scrRect.extent.y = 1;
	WinCopyRectangle(tmpWnd2, tmpWnd3, &scrRect, 4, nTop + 1, winPaint);
	scrRect.topLeft.x = 3;
	scrRect.topLeft.y = nTop - (scrHeight - dstHeight) + 2;
	scrRect.extent.x = scrWidth - 6;
	scrRect.extent.y = 1;
	WinCopyRectangle(tmpWnd2, tmpWnd3, &scrRect, 3, nTop + 2, winPaint);
	scrRect.topLeft.x = 2;
	scrRect.topLeft.y = nTop - (scrHeight - dstHeight) + 3;
	scrRect.extent.x = scrWidth - 4;
	scrRect.extent.y = 1;
	WinCopyRectangle(tmpWnd2, tmpWnd3, &scrRect, 2, nTop + 3, winPaint);
	scrRect.topLeft.x = 1;
	scrRect.topLeft.y = nTop - (scrHeight - dstHeight) + 4;
	scrRect.extent.x = scrWidth - 2;
	scrRect.extent.y = 2;
	WinCopyRectangle(tmpWnd2, tmpWnd3, &scrRect, 1, nTop + 4, winPaint);

	if (pPrefs->uBackground == 1)
	{
		scrRect.topLeft.x = 2;
		scrRect.topLeft.y = nTop;
		scrRect.extent.x  = scrWidth-4;
		scrRect.extent.y  = dstHeight-2;
		WinDrawRectangleFrame(roundFrame,&scrRect);		
	}
	else if (pPrefs->uBackground == 3)
	{
		scrRect.topLeft.x = 5;
		scrRect.topLeft.y = nTop+5;
		scrRect.extent.x  = 130;
		scrRect.extent.y  = 65;
		WinSetForeColorRGB(&rgbBlack, NULL);
		WinDrawRectangle(&scrRect, 10);		
	}

	// switch to normal coords
	if (attr == kDensityDouble)
	{
		WinSetCoordinateSystem(kCoordinatesStandard);
		nTop >>= 1;
	}


/*
 5	time				battery
20	date				IR/BT
40	ram					mute switch
55	db					messages
70	card				phone
85	number/operator		traffic
*/
	// 3. draw the gadgets
	WinSetDrawMode(winOverlay);
	FntSetFont(stdFont);
	{
		Char string[64], date[32], *ptr;
		UInt32 now = TimGetSeconds();
		DateTimeType dt;
		UInt32 romVersion = 0;
		UInt32 free, max;
		UInt16 volt, warnThreshold, criticalThreshold;
		UInt16 size;
		UInt8 percent = 0;
		ExgSocketType es;
		UInt16 volRefNum = 0;
		UInt32 volIterator = vfsIteratorStart;
		Boolean bCardPresent = false;
		Boolean bCardWritable = false;
		RGBColorType rgbBattery;

		FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);

		// time
		FntSetFont(ledFont);//largeBoldFont);
		TimSecondsToDateTime(now, &dt);
		TimeToAscii(dt.hour, dt.minute, PrefGetPreference(prefTimeFormat), string);
		// find the char between hour and minute
		ptr = StrChr(string, ':');
		if (ptr == NULL)
			ptr = StrChr(string, '.'); // look for '.'
		if (ptr == NULL)
			ptr = StrChr(string, ','); // lok for ','
		// blinking colon (:)
		if (ptr)
		{
			Int16 left = 30 - (FntLineWidth(ptr+1, StrLen(ptr+1)) - FntLineWidth(string, ptr-string)) / 2;
			RenderMyText(string, ptr-string, left - FntLineWidth(string, ptr-string), nTop + 3, 255, 255, 255, bWhite, !bWhite);
			FntSetFont(largeBoldFont);
			if ((now & 0x01) == 0x00)
				RenderMyText(ptr, 1, left + 1, nTop + 5, 255, 255, 255, bWhite, !bWhite);
			FntSetFont(ledFont);
			RenderMyText(ptr+1, StrLen(ptr+1), left + 5, nTop + 3, 255, 255, 255, bWhite, !bWhite);
		}
		else
		{
			RenderMyText(string, StrLen(string), 34 - FntLineWidth(string, StrLen(string)) / 2, nTop + 5, 255, 255, 255, bWhite, !bWhite);
		}
		FntSetFont(stdFont);

		// date
		DateToAscii(dt.month, dt.day, dt.year, PrefGetPreference(prefDateFormat), date);
		DateTemplateToAscii("^1r, ", dt.month, dt.day, dt.year, string, sizeof(string));
		StrNCat(string, date, sizeof(string));
		string[sizeof(string) - 1] = 0;
		RenderMyText(string, StrLen(string), 35 - FntLineWidth(string, StrLen(string)) / 2, nTop + 22, 255, 255, 255, bWhite, !bWhite);

		StrPrintF(string, "wk");
		RenderMyText(string, StrLen(string), 60 - FntLineWidth(string, StrLen(string)) / 2, nTop + 2, 255, 255, 255, bWhite, !bWhite);
		StrPrintF(string, "%d", GetWeekNumber(dt.month, dt.day, dt.year));
		RenderMyText(string, StrLen(string), 60 - FntLineWidth(string, StrLen(string)) / 2, nTop + 10, 255, 255, 255, bWhite, !bWhite);

		// memory, dynamic
		MemHeapFreeBytes(0, &free, &max);
		StrPrintF(string, "RAM %ld.%ld/%ld.%ld MB", (free>>10) / 1000, ((free>>10) / 100) % 10, (max>>10) / 1000, ((max>>10) / 100) % 10);
		RenderMyText(string, StrLen(string), 5, nTop + 40, 255, 255, 255, bWhite, !bWhite);
		if (romVersion >= 0x05000000)
			MemHeapFreeBytes(0x8001, &free, &max);
		else
			MemHeapFreeBytes(0x0001, &free, &max);
		StrPrintF(string, "DB %ld.%ld/%ld.%ld MB", (free>>10) / 1000, ((free>>10) / 100) % 10, (max>>10) / 1000, ((max>>10) / 100) % 10);
		RenderMyText(string, StrLen(string), 5, nTop + 55, 255, 255, 255, bWhite, !bWhite);

		// Battery
		volt = SysBatteryInfo(false, &warnThreshold, &criticalThreshold, NULL, NULL, NULL, &percent);
		if ((volt < criticalThreshold) || (percent < 5))
		{
			MemMove(&rgbBattery, &rgbRed, sizeof(rgbBattery));
		}
		else if ((volt < warnThreshold) || (percent < 20))
		{
			MemMove(&rgbBattery, &rgbYellow, sizeof(rgbBattery));
		}
		else
		{
			MemMove(&rgbBattery, &rgbGreen, sizeof(rgbBattery));
		}
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 6, 80 + 75 * percent / 100, nTop + 6);
		rgbBattery.r = maximum(0, rgbBattery.r-16);
		rgbBattery.g = maximum(0, rgbBattery.g-16);
		rgbBattery.b = maximum(0, rgbBattery.b-16);
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 7, 80 + 75 * percent / 100, nTop + 7);
		rgbBattery.r = maximum(0, rgbBattery.r-8);
		rgbBattery.g = maximum(0, rgbBattery.g-8);
		rgbBattery.b = maximum(0, rgbBattery.b-8);
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 8, 80 + 75 * percent / 100, nTop + 8);
		rgbBattery.r = maximum(0, rgbBattery.r-4);
		rgbBattery.g = maximum(0, rgbBattery.g-4);
		rgbBattery.b = maximum(0, rgbBattery.b-4);
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 9, 80 + 75 * percent / 100, nTop + 9);
		rgbBattery.r = maximum(0, rgbBattery.r-2);
		rgbBattery.g = maximum(0, rgbBattery.g-2);
		rgbBattery.b = maximum(0, rgbBattery.b-2);
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 10, 81 + 75 * percent / 100, nTop + 10);
		WinDrawLine(79, nTop + 11, 81 + 75 * percent / 100, nTop + 11);
		rgbBattery.r = maximum(0, rgbBattery.r-4);
		rgbBattery.g = maximum(0, rgbBattery.g-4);
		rgbBattery.b = maximum(0, rgbBattery.b-4);
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 12, 80 + 75 * percent / 100, nTop + 12);
		rgbBattery.r = maximum(0, rgbBattery.r-8);
		rgbBattery.g = maximum(0, rgbBattery.g-8);
		rgbBattery.b = maximum(0, rgbBattery.b-8);
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 13, 80 + 75 * percent / 100, nTop + 13);
		rgbBattery.r = maximum(0, rgbBattery.r-16);
		rgbBattery.g = maximum(0, rgbBattery.g-16);
		rgbBattery.b = maximum(0, rgbBattery.b-16);
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 14, 80 + 75 * percent / 100, nTop + 14);
		rgbBattery.r = maximum(0, rgbBattery.r-32);
		rgbBattery.g = maximum(0, rgbBattery.g-32);
		rgbBattery.b = maximum(0, rgbBattery.b-32);
		WinSetForeColorRGB(&rgbBattery, NULL);
		WinDrawLine(79, nTop + 15, 80 + 75 * percent / 100, nTop + 15);
		WinSetForeColorRGB(&rgbWhite, NULL);
		WinDrawLine(79, nTop + 5, 80 + 75, nTop + 5);
		WinSetForeColorRGB(&rgbBlack, NULL);
		WinDrawLine(79, nTop + 16, 80 + 75, nTop + 16);
		StrCopy(string, "Battery");
		RenderMyText(string, StrLen(string), 85, nTop + 5, 255, 255, 255, bWhite, !bWhite);
		FntSetFont(boldFont);
		StrPrintF(string, "%d%%", percent);
		if ((volt < criticalThreshold) || (percent < 5))
		{
			RenderMyText(string, StrLen(string), 150 - FntLineWidth(string, StrLen(string)), nTop + 5, 255, 0, 0, false, !bWhite);
		}
		else if ((volt < warnThreshold) || (percent < 20))
		{
			RenderMyText(string, StrLen(string), 150 - FntLineWidth(string, StrLen(string)), nTop + 5, 255, 255, 0, false, !bWhite);
		}
		else
		{
			RenderMyText(string, StrLen(string), 150 - FntLineWidth(string, StrLen(string)), nTop + 5, 255, 255, 255, bWhite, !bWhite);
		}
		FntSetFont(stdFont);

		// IR Beaming
		if (SysLibFind(irLibName, &es.libraryRef) == errNone)
		{
			Boolean bEnable = true;
			UInt16 size = sizeof(Boolean);
			ExgControl(&es, irGetScanningMode, (void *) &bEnable, &size);
			StrCopy(string, "IR");
			RenderMyText(string, StrLen(string), 80, nTop + 20, 255, 255, 255, bWhite, !bWhite);
			if (bEnable)
			{
				StrCopy(string, "On");
				RenderMyText(string, StrLen(string), 95, nTop + 20, 255, 255, 0, false, true);
			}
			else
			{
				StrCopy(string, "Off");
				RenderMyText(string, StrLen(string), 95, nTop + 20, 0, 255, 0, false, !bWhite);
			}
		}

		// BT
		size = 0;
		if (PrefGetAppPreferences('abtp', 1, NULL, &size, true) != noPreferenceFound)
		{
			UInt8 *ptr = MemPtrNew(size);
			if (ptr)
			{
				PrefGetAppPreferences('abtp', 1, ptr, &size, true);
				StrCopy(string, "BT");
				RenderMyText(string, StrLen(string), 120, nTop + 20, 255, 255, 255, bWhite, !bWhite);
				if (ptr[0])
				{
					StrCopy(string, "On");
					RenderMyText(string, StrLen(string), 135, nTop + 20, 255, 255, 0, false, true);
				}
				else
				{
					StrCopy(string, "Off");
					RenderMyText(string, StrLen(string), 135, nTop + 20, 0, 255, 0, false, !bWhite);
				}
				MemPtrFree(ptr);
			}
		}
		
		// SD-Card
		while (volIterator != vfsIteratorStop)
		{
			if (VFSVolumeEnumerate(&volRefNum, &volIterator) == errNone)
			{
				// Do something with the volRefNum
				VolumeInfoType info;
				VFSVolumeInfo(volRefNum, &info);
				if ((info.attributes & vfsVolumeAttrNonRemovable) == 0)
				{
					bCardPresent = true;
					if (vfsVolumeAttrReadOnly & vfsVolumeAttrReadOnly)
					{
						bCardWritable = true;
					}
					break;
				}
			}
			else
			{
				break;
			}
		}
		StrCopy(string, "Card");
		RenderMyText(string, StrLen(string), 5, nTop + 69, 255, 255, 255, bWhite, !bWhite);
		if (bCardPresent)
		{
			if (bCardWritable)
			{
				StrCopy(string, "Writable");
				RenderMyText(string, StrLen(string), 30, nTop + 69, 0, 255, 0, false, !bWhite);
			}
			else
			{
				StrCopy(string, "Read Only");
				RenderMyText(string, StrLen(string), 30, nTop + 69, 255, 255, 0, false, true);
			}
		}
		else
		{
			StrCopy(string, "Missing");
			RenderMyText(string, StrLen(string), 30, nTop + 70, 255, 0, 0, false, !bWhite);
		}
		
		// current Traffic from 2LaunchMe
		{
			struct tagExportStruct *ptrExport = NULL;
			if ((FtrGet('2LMe', 0x1234, (UInt32 *) &ptrExport) == errNone) && ptrExport)
			{
				if (ptrExport->dwVersion > 0)
				{
					Int16 count, width;
					Boolean bFit;
					UInt8 uInfoSelection = 0;
					if (ptrExport->dwVersion > 1)
						uInfoSelection = ptrExport->uInfoSelection;
					
					StrPrintF(string, "%s", ptrExport->name);
					count = StrLen(string);
					if (uInfoSelection != 0)
						width = 60;
					else
						width = 40;
					FntCharsInWidth(string, &width, &count, &bFit);
					if (!bFit)
						string[count] = 0;
					RenderMyText(string, StrLen(string), 80, nTop + 83, 255, 255, 255, bWhite, !bWhite);
					if (uInfoSelection == 1)
					{
						StrPrintF(string, "%ld.%01ldK", (Int32) ((ptrExport->dwDataIn + ptrExport->dwDataOut) / 1000), (Int32) (((ptrExport->dwDataIn + ptrExport->dwDataOut) / 100) % 10));
						RenderMyText(string, StrLen(string), 160 - FntLineWidth(string, StrLen(string)), nTop + 83, 255, 255, 255, bWhite, !bWhite);
					}
					else if (uInfoSelection == 2)
					{
						StrPrintF(string, "%ld:%02lds", (Int32) (ptrExport->dwDuration / 60), (Int32) (ptrExport->dwDuration % 60));
						RenderMyText(string, StrLen(string), 160 - FntLineWidth(string, StrLen(string)), nTop + 83, 255, 255, 255, bWhite, !bWhite);
					}
					else
					{
						StrPrintF(string, "%ldK", (Int32) ((ptrExport->dwDataIn + ptrExport->dwDataOut) >> 10));
						RenderMyText(string, StrLen(string), 140 - FntLineWidth(string, StrLen(string)), nTop + 83, 255, 255, 255, bWhite, !bWhite);
						StrPrintF(string, "%ldm", (Int32) (ptrExport->dwDuration / 60));
						RenderMyText(string, StrLen(string), 160 - FntLineWidth(string, StrLen(string)), nTop + 83, 255, 255, 255, bWhite, !bWhite);
					}
				}
			}
		}


		// Mute switch on Treo
		FntSetFont(boldFont);
		if (CheckForTreo())
		{
			UInt16 refNum;

			if (ptrPhoneState)
			{
				StrCopy(string, ptrPhoneState);
				RenderMyText(string, StrLen(string), 155 - FntLineWidth(string, StrLen(string)), nTop + 40, 255, 0, 0, false, !bWhite);
			}
			else
			{
				UInt32 muteSwitch;

				HsAttrGet(hsAttrRingSwitch, 0, &muteSwitch);
				if (muteSwitch == 0)
				{
					StrCopy(string, "MUTE");
					RenderMyText(string, StrLen(string), 155 - FntLineWidth(string, StrLen(string)), nTop + 40, 255, 128, 0, false, !bWhite);
				}
				else
				{
					StrCopy(string, "LOUD");
					RenderMyText(string, StrLen(string), 155 - FntLineWidth(string, StrLen(string)), nTop + 40, 0, 255, 0, false, !bWhite);
				}
			}

			FntSetFont(stdFont);
			if (HsGetPhoneLibrary(&refNum) == errNone)
			{
				if (PhnLibOpen(refNum) == errNone)
				{
					CharPtr ptr = NULL;
					PhnAddressList aList;
					PhnAddressHandle address;
					PhnMsgBoxDataType boxdata;

					// number/operator alternating
					if (TimGetSeconds() & 0x01)
					{
						if (PhnLibModulePowered(refNum))
						{
							PhnOperatorID id;
							GSMRegistrationMode mode;
							if ((PhnLibCurrentOperator(refNum, &id, &ptr, &mode) == errNone) && ptr)
							{
								RenderMyText(ptr, StrLen(ptr), 5, nTop + 83, 255, 255, 255, bWhite, !bWhite);
							}
						}
					}
					else
					{
						if (PhnLibGetOwnNumbers(refNum, &aList) == errNone)
						{
							UInt32 dwPhoneType;

							HsAttrGet (hsAttrPhoneType, 0, &dwPhoneType);
							if (dwPhoneType != hsAttrPhoneTypeCDMA)
							{
								if (PhnLibGetNth(refNum, aList, 1, &address) == errNone)
								{
									ptr = PhnLibGetField(refNum, address, phnAddrFldPhone);
									if (ptr)
									{
										RenderMyText(ptr, StrLen(ptr), 5, nTop + 83, 255, 255, 255, bWhite, !bWhite);
										MemPtrFree(ptr);
									}
									MemHandleFree(address);
								}
							}
							else
							{
								if (PhnLibAPGetNth(refNum, aList, 1, &address) == errNone)
								{
									ptr = PhnLibAPGetField(refNum, address, phnAddrFldPhone);
									if (ptr)
									{
										RenderMyText(ptr, StrLen(ptr), 5, nTop + 83, 255, 255, 255, bWhite, !bWhite);
										MemPtrFree(ptr);
									}
									MemHandleFree(address);
								}
							}
							MemHandleFree(aList);
						}
					}
					if (PhnLibBoxInformation(refNum, &boxdata) == errNone)
					{
						if (boxdata.indicatorOn)
						{
							ptr = "Messages";
							RenderMyText(ptr, StrLen(ptr), 155 - FntLineWidth(ptr, StrLen(ptr)), nTop + 55, 255, 255, 255, bWhite, !bWhite);
						}
					}
					if (PhnLibModulePowered(refNum))
					{
						UInt32 dwPhoneType;
						UInt16 quality = 0;
						PhnLibSignalQuality(refNum, &quality);
						StrPrintF(string, "Phone (%d)", quality);
						RenderMyText(string, StrLen(string), 135 - FntLineWidth(string, StrLen(string)), nTop + 70, 255, 255, 255, bWhite, !bWhite);

						HsAttrGet (hsAttrPhoneType, 0, &dwPhoneType);
						if (dwPhoneType != hsAttrPhoneTypeCDMA)
						{
							if (quality > 4)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(140, nTop + 70 + 8, 140, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(141, nTop + 70 + 8, 141, nTop + 70 + 10);
							}
							if (quality > 8)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(143, nTop + 70 + 6, 143, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(144, nTop + 70 + 6, 144, nTop + 70 + 10);
							}
							if (quality > 12)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(146, nTop + 70 + 4, 146, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(147, nTop + 70 + 4, 147, nTop + 70 + 10);
							}
							if (quality > 16)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(149, nTop + 70 + 2, 149, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(150, nTop + 70 + 2, 150, nTop + 70 + 10);
							}
							if (quality > 20)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(152, nTop + 70 + 0, 152, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(153, nTop + 70 + 0, 153, nTop + 70 + 10);
							}
							if (PhnLibGPRSAttached(refNum))
							{
								RectangleType rect;
								rect.topLeft.x = 140;
								rect.topLeft.y = nTop + 70;
								rect.extent.x = 5;
								rect.extent.y = 5;
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawRectangle(&rect, 0);
								rect.topLeft.x = 141;
								rect.topLeft.y = nTop + 71;
								rect.extent.x = 3;
								rect.extent.y = 3;
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawRectangle(&rect, 0);
							}
						}
						else if (quality > 0)
						{
							if (quality < 105)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(140, nTop + 70 + 8, 140, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(141, nTop + 70 + 8, 141, nTop + 70 + 10);
							}
							if (quality < 95)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(143, nTop + 70 + 6, 143, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(144, nTop + 70 + 6, 144, nTop + 70 + 10);
							}
							if (quality < 85)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(146, nTop + 70 + 4, 146, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(147, nTop + 70 + 4, 147, nTop + 70 + 10);
							}
							if (quality < 75)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(149, nTop + 70 + 2, 149, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(150, nTop + 70 + 2, 150, nTop + 70 + 10);
							}
							if (quality < 65)
							{
								WinSetForeColorRGB(&rgbBlack, NULL);
								WinDrawLine(152, nTop + 70 + 0, 152, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(153, nTop + 70 + 0, 153, nTop + 70 + 10);
							}
						}
						{
							PhnRoamStatus stat;
							PhnLibRoaming(refNum, &stat);
							if (stat != PhnRoamStatusOff)
							{
								WinSetForeColorRGB(&rgbRed, NULL);
								WinDrawLine(155, nTop + 70 + 0, 152, nTop + 70 + 10);
								WinSetForeColorRGB(&rgbWhite, NULL);
								WinDrawLine(156, nTop + 70 + 0, 153, nTop + 70 + 10);
							}
						}
					}
					else
					{
						StrCopy(string, "Phone Off");
						RenderMyText(string, StrLen(string), 155 - FntLineWidth(string, StrLen(string)), nTop + 70, 255, 0, 0, false, !bWhite);
					}
					PhnLibClose(refNum);
				}
			}
		}

		//#### Added by Shimon Shnitzer: draw 4cast weather info line (if exists) ############
		if (pPrefs->u4Cast)
		{
			if (NeedToDraw4cast())
				Update4cast(nTop+100, bWhite, pPrefs);
			else if (NeedToDrawKMeteo())
				UpdateKMeteo(nTop+100, bWhite, pPrefs);
		}

	}
	WinPopDrawState();

	// 4. draw our screen image
	scrRect.topLeft.x = 0;
	scrRect.topLeft.y = 0;
	scrRect.extent.x = scrWidth;
	scrRect.extent.y = scrHeight;
	WinCopyRectangle(tmpWnd3, scrWnd, &scrRect, 0, 0, winPaint);
}




static
Boolean HandleKeyGuard()
{
	Err error;
	BitmapPtr scrBmp = NULL;
	WinHandle scrWnd = NULL;
	BitmapPtr tmpBmp1 = NULL;
	BitmapPtrV3 tmp3mp1 = NULL;
	WinHandle tmpWnd1 = NULL;
	BitmapPtr tmpBmp2 = NULL;
	BitmapPtrV3 tmp3mp2 = NULL;
	WinHandle tmpWnd2 = NULL;
	WinHandle tmpWnd3 = NULL;
	Coord scrWidth, scrHeight;
	UInt8 scrDepth;
	UInt16 scrBPP;
	WinHandle wndDraw, wndActive;
	tagPrefs locPrefs;
	UInt32 attr, value = 0;
	Boolean result = false;
	UInt16 oldCoordSys;

// FIXME: Debug
FileHand hdlGPSLog = NULL;

	WinScreenGetAttribute(winScreenDensity, &attr);

	ReadPrefs(&locPrefs);
	if (!locPrefs.bEnabled)
		return false;

	// still displayed?
	if ((FtrGet(MyAppID, 1, &value) == errNone) && value)
		return true;
	FtrSet(MyAppID, 1, 1);
	// remove the keyguard still active flag
	FtrUnregister(MyAppID, 2);

	if (CheckForTreo())
	{
		UInt32 keyguard = (UInt32) -1;
		HsAttrSet(hsAttrKeyboardLocked, 0, &keyguard);
	}

// FIXME: Debug
{
Char strFileName[32];
DateTimeType dt;
TimSecondsToDateTime(TimGetSeconds(), &dt);
StrPrintF(strFileName, "NxVe_%04d%02d%02d_%02d%02d%02d.nmea", dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second);
//hdlGPSLog = FileOpen(0, strFileName, 0, MyAppID, fileModeReadWrite, NULL);
}

	EvtResetAutoOffTimer();
	WinPushDrawState();

	oldCoordSys = WinGetCoordinateSystem();

	wndDraw = WinGetDrawWindow();
	wndActive = WinGetActiveWindow();

	// let's make a truecolor copy of the screen
	scrWnd = WinGetDisplayWindow();
	if (scrWnd)
	{
		scrBmp = WinGetBitmap(scrWnd);
		if (scrBmp)
		{
			BmpGetDimensions(scrBmp, &scrWidth, &scrHeight, &scrBPP);
			scrDepth = BmpGetBitDepth(scrBmp);

HostTraceOutputTL(appErrorClass, "%d %d %d", scrWidth, scrHeight, scrBPP);
//				scrWidth /= 2;
//				scrHeight /= 2;
			tmpBmp1 = BmpCreate(scrWidth, scrHeight, 16, NULL, &error);
			if (tmpBmp1)
			{
				if (attr == kDensityDouble)
					tmp3mp1 = BmpCreateBitmapV3(tmpBmp1, kCoordinatesDouble, BmpGetBits(tmpBmp1), NULL);
				else
					tmp3mp1 = BmpCreateBitmapV3(tmpBmp1, kCoordinatesStandard, BmpGetBits(tmpBmp1), NULL);
				if (tmp3mp1)
				{
					tmpWnd1 = WinCreateBitmapWindow((BitmapPtr) tmp3mp1, &error);
					if (tmpWnd1)
					{
						RectangleType scrRect;
						Coord dstHeight;

						if (attr == kDensityDouble)
							WinSetCoordinateSystem(kCoordinatesDouble);

						scrRect.topLeft.x = 0;
						scrRect.topLeft.y = 0;
						scrRect.extent.x = scrWidth;
						scrRect.extent.y = scrHeight;
						WinCopyRectangle(scrWnd, tmpWnd1, &scrRect, 0, 0, winPaint);
						
						// make a copy of the lower part, half transparent
						if (scrHeight < 320)
							dstHeight = scrHeight - 55;
						else
							dstHeight = scrHeight - 110;
//						dstHeight = scrHeight * 2 / 3;

						//#### Added by Shimon Shnitzer: draw 4cast weather info line (if exists) #########
						if (locPrefs.u4Cast && (NeedToDraw4cast() || NeedToDrawKMeteo()))
							dstHeight += 30;

						tmpBmp2 = BmpCreate(scrWidth, dstHeight, 16, NULL, &error);
						if (tmpBmp2)
						{
							if (attr == kDensityDouble)
								tmp3mp2 = BmpCreateBitmapV3(tmpBmp2, kCoordinatesDouble, BmpGetBits(tmpBmp2), NULL);
							else
								tmp3mp2 = BmpCreateBitmapV3(tmpBmp2, kCoordinatesStandard, BmpGetBits(tmpBmp2), NULL);
							if (tmp3mp2)
							{
								tmpWnd2 = WinCreateBitmapWindow((BitmapPtr) tmp3mp2, &error);
								if (tmpWnd2)
								{
									Int32 x, y;

									UInt16 *tmpBits1 = BmpGetBits(tmpBmp1);
									UInt16 *tmpBits2 = BmpGetBits(tmpBmp2);

									if (scrWidth == 320)
									{
										Int32 y1 = 0;
										Int32 y2 = (scrHeight - dstHeight) * (Int32) scrWidth;
										if (locPrefs.uBackground == 1)
										{
											// White
											for (y=0; y<dstHeight; y++)
											{
												for (x=0; x<scrWidth; x++)
													tmpBits2[y1 + x] = 0xffff;
												y1 += scrWidth;
												y2 += scrWidth;
											}
										}
										else if (locPrefs.uBackground == 2)
										{
											// Black
											for (y=0; y<dstHeight; y++)
											{
												for (x=0; x<scrWidth; x++)
													tmpBits2[y1 + x] = 0;
												y1 += scrWidth;
												y2 += scrWidth;
											}
										}
										else
										{
											// Transparent
											fastcopy(tmpBits1, scrWidth, scrHeight, tmpBits2, dstHeight);
										}
									}
									else
									{
										Int32 y1 = 0;
										Int32 y2 = (scrHeight - dstHeight) * (Int32) scrWidth;
										for (y=0; y<dstHeight; y++)
										{
//											Int32 y1 = y * scrWidth;
//											Int32 y2 = ((scrHeight - dstHeight) + y) * scrWidth;
											switch (locPrefs.uBackground)
											{
												case 1:	// White
													// mode is 5 - 6 - 5
													for (x=0; x<scrWidth; x++)
														tmpBits2[y1 + x] = 0xffff;
													break;
												case 2:	// Black
													// mode is 5 - 6 - 5
													for (x=0; x<scrWidth; x++)
														tmpBits2[y1 + x] = 0;
													break;
												default:	// Transparent
													// mode is 5 - 6 - 5
													for (x=0; x<scrWidth; x++)
														tmpBits2[y1 + x] = (tmpBits1[y2 + x] >> 1) & 0x7bef;
												break;
											}
											y1 += scrWidth;
											y2 += scrWidth;
										}
									}
/*
									if (locPrefs.uBackground == 3)
									{
										for (y=5; y<65; y++)
										{
											Int32 y1 = y * scrWidth;
											Int32 y2 = ((scrHeight - dstHeight) + y) * scrWidth;
											for (x=5; x<130; x++)
											{
												tmpBits2[y1 + x] = 0;
											}
										}
									}
*/
									// create our draw window
									tmpWnd3 = WinCreateOffscreenWindow(scrWidth, scrHeight, nativeFormat, &error);
									if (tmpWnd3)
									{
										Int16 nOldScreenY = 0;
										Boolean bOldPenDown = false;
										Int16 nTop = scrHeight - dstHeight;
										UInt32 dwInactivityTimer = TimGetSeconds();
										Char *ptrPhoneState = NULL;
										Int32 nEventTimeOut = SysTicksPerSecond();

										if (CheckForTreo())
										{
											UInt16 refNum;

											if (HsGetPhoneLibrary(&refNum) == errNone)
											{
												if (PhnLibOpen(refNum) == errNone)
												{
													UInt32 dwPhoneType;

													HsAttrGet (hsAttrPhoneType, 0, &dwPhoneType);
													// no fwd for CDMA phones
													if (dwPhoneType != hsAttrPhoneTypeCDMA)
													{
														PhnLineStateType pls;
														pls.version = 1;
														pls.size    = sizeof(PhnLineStateType);
														if (PhnLibGetLineState(refNum, 1, &pls)==errNone)
														{
															if (pls.divertIndicator)
																ptrPhoneState = "FWRD";
														}
													}

													PhnLibClose(refNum);
												}
											}
										}
										
										result = true;

										WinSetCoordinateSystem(oldCoordSys);

										while (1)
										{
											EventType event;
											Int16 nScreenX, nScreenY;
											Int16 nHighScreenX, nHighScreenY;
											Boolean bPenDown;
											Boolean bResult;
											UInt32 dwLastMove = 0;
											UInt32 value;

											if (attr == kDensityDouble)
												WinSetCoordinateSystem(oldCoordSys);
											EvtGetEvent(&event, nEventTimeOut);
											if (attr == kDensityDouble)
												WinSetCoordinateSystem(kCoordinatesDouble);

											// leave it if the device goes to sleep again...
											if (event.eType == keyDownEvent)
											{
// FIXME: Debug
if (hdlGPSLog)
{
Char buffer[16];
StrPrintF(buffer, "key: %x\r\n", event.data.keyDown.chr);
FileWrite(hdlGPSLog, buffer, 1, StrLen(buffer), NULL);
}

												if (event.data.keyDown.chr == vchrPowerOff)
												{
													// remember: keyguard still active
													FtrSet(MyAppID, 2, 1);
													// we still need to send it to the System
													SysHandleEvent(&event);
													break;
												}
												else if ((locPrefs.bCenterButton && (/*event.data.keyDown.chr == 0x0136)*/NavSelectPressed(&event))) || 
														 (locPrefs.bPageDownButton && ((event.data.keyDown.chr == vchrPageDown) || (event.data.keyDown.chr == vchrRockerDown))) ||
														 (event.data.keyDown.chr == vchrHardCradle))
												{
													Boolean bOptionPressed = true;
													if (CheckForTreo())
													{
														UInt32 keys[3] = {0};
														HsKeyCurrentStateExt(keys);
														if ((keys[1] & keyBitExt1Opt) == 0)
															bOptionPressed = false;
													}
													if (!locPrefs.bOptionKey || bOptionPressed ||
														(event.data.keyDown.chr == vchrHardCradle))
													{
														// we got a center or down key
														int i = 0;
														int speed = 2;

														dwLastMove = TimGetTicks();
														while (nTop < scrHeight)
														{
															nTop += (TimGetTicks() - dwLastMove) * speed;
															dwLastMove = TimGetTicks();
															DoTheUpdate(ptrPhoneState, scrWidth, scrHeight, dstHeight, nTop+i, tmpWnd1, tmpWnd2, tmpWnd3, scrWnd, &locPrefs);
															i++;
															if ((i & 0x03) == 0x01)
																speed<<=1;
														}

														if (CheckForTreo())
														{
															UInt32 keyguard = 0;
															HsAttrSet(hsAttrKeyboardLocked, 0, &keyguard);
														}
														
														if (event.data.keyDown.chr == vchrHardCradle)
														{
															// we still need to send it to the System
															SysHandleEvent(&event);
														}

														break;
													}
												}
												if ((event.data.keyDown.chr != vchrHardPower) &&
													(event.data.keyDown.chr != vchrAutoOff) &&
													(event.data.keyDown.chr != vchrAlarm) &&
													(event.data.keyDown.chr != hsChrWakeup) &&
													(event.data.keyDown.chr != hsChrKeyboardLock))
													continue;
											}

											// ensure, SysHandleEvent is in standard coords...
											if (attr == kDensityDouble)
												WinSetCoordinateSystem(oldCoordSys);
											bResult = SysHandleEvent(&event);
											if (attr == kDensityDouble)
												WinSetCoordinateSystem(kCoordinatesDouble);
											if (bResult)
												continue;

											EvtGetPen(&nScreenX, &nScreenY, &bPenDown);
											nHighScreenX = nScreenX;
											nHighScreenY = nScreenY;
											if (attr == kDensityDouble)
											{
												nHighScreenX = nScreenX * 2;
												nHighScreenY = nScreenY * 2;
											}
										
											if (!bPenDown)
											{
												int i;
												int speed = 1;

												for (i=0; (nTop-i)<(scrHeight - dstHeight); i+=speed)
												{
													DoTheUpdate(ptrPhoneState, scrWidth, scrHeight, dstHeight, nTop-i, tmpWnd1, tmpWnd2, tmpWnd3, scrWnd, &locPrefs);
													if ((i & 0x03) == 0x01)
														speed<<=1;
													SysTaskDelay(1);
												}
											
												nTop = scrHeight - dstHeight;
											}
											else if (bOldPenDown)
											{
												if (!locPrefs.bWipeOnClock
													|| ((nHighScreenX > 5) && (nHighScreenX < 135) && (nHighScreenY > (nTop+5)) && (nHighScreenY < (nTop+70))))
												{
													dwInactivityTimer = TimGetSeconds();
													nTop -= ((nOldScreenY - nScreenY) << 1);
													if (nTop < (scrHeight - dstHeight))
													{
														nTop = scrHeight - dstHeight;
													}
													else if (nTop > (scrHeight - dstHeight + 80))//dstHeight)
													{
														int i = 0;
														int speed = 2;

														dwLastMove = TimGetTicks();
														while (nTop < scrHeight)
														{
															nTop += (TimGetTicks() - dwLastMove) * speed;
															dwLastMove = TimGetTicks();
															DoTheUpdate(ptrPhoneState, scrWidth, scrHeight, dstHeight, nTop+i, tmpWnd1, tmpWnd2, tmpWnd3, scrWnd, &locPrefs);
															i++;
															if ((i & 0x03) == 0x01)
																speed <<= 1;
														}

														if (CheckForTreo())
														{
															UInt32 keyguard = 0;
															HsAttrSet(hsAttrKeyboardLocked, 0, &keyguard);
														}

														break;
													}
												}
											}
											nOldScreenY = nScreenY;
											bOldPenDown = bPenDown;
										
											DoTheUpdate(ptrPhoneState, scrWidth, scrHeight, dstHeight, nTop, tmpWnd1, tmpWnd2, tmpWnd3, scrWnd, &locPrefs);

											// 5 seconds inactivity?
											if ((TimGetSeconds() - dwInactivityTimer) > locPrefs.nTimeOut)
											{
												EvtEnqueueKey(vchrAutoOff, 0, commandKeyMask);
											}

WinSetDrawWindow(scrWnd);
/*
											if (CheckForTreo())
											{
												Char strTemp[32];
												UInt32 keyguard = 1;
												HsAttrGet(hsAttrKeyboardLocked, 0, &keyguard);
												StrPrintF(strTemp, "%ld", keyguard);
												WinDrawChars(strTemp, StrLen(strTemp), 10, 80);
											}
*/
											// check for Ftr to dismiss by 2PlayMe
											if ((FtrGet('NxVe', 'Hk', &value) == errNone) && (value == 1))
											{
												FtrUnregister('NxVe', 'Hk');
												if (CheckForTreo())
												{
													UInt32 keyguard = 0;
													HsAttrSet(hsAttrKeyboardLocked, 0, &keyguard);
												}
												break;
											}
										}

										WinDeleteWindow(tmpWnd3, false);
									}
									WinDeleteWindow(tmpWnd2, false);
								}
								BmpDelete((BitmapPtr) tmp3mp2);
							}
							BmpDelete(tmpBmp2);
						}

						if (attr == kDensityDouble)
							WinSetCoordinateSystem(kCoordinatesDouble);

						// restore screen
						scrRect.topLeft.x = 0;
						scrRect.topLeft.y = 0;
						scrRect.extent.x = scrWidth;
						scrRect.extent.y = scrHeight;
						WinCopyRectangle(tmpWnd1, scrWnd, &scrRect, 0, 0, winPaint);

						WinSetCoordinateSystem(oldCoordSys);
						
						WinDeleteWindow(tmpWnd1, false);
					}
					BmpDelete((BitmapPtr) tmp3mp1);
				}
				BmpDelete(tmpBmp1);
			}
		}
	}
	WinPopDrawState();
	WinSetActiveWindow(wndActive);
	WinSetDrawWindow(wndDraw);

// FIXME: Debug
if (hdlGPSLog)
{
FileClose(hdlGPSLog);
hdlGPSLog = NULL;
}

	FtrUnregister(MyAppID, 1);

	return result;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the Puzzle 
 *              application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    0
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			vmk	8/19/95	Initial Revision
 *			hjo	02/07/02	
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	UInt16 error;

	HostTraceInit();
	
	error = RomVersionCompatible (version50, launchFlags);
	if (error)
		return (error);

	if ((cmd == sysAppLaunchCmdNormalLaunch) ||
		(cmd == sysAppLaunchCmdPanelCalledFromApp) ||
		(cmd == sysAppLaunchCmdReturnFromPanel))
	{
		// Store call mode and save current panel
		PanelSetCallMode(cmd, MyAppID);

		// normal start
		error = StartApplication();	// Application start code
		if (error)
			return error;

		EventLoop();	// Event loop

		StopApplication ();	// Application stop code
	}
	else if ((cmd == sysAppLaunchCmdSystemReset) ||
		(cmd == sysAppLaunchCmdSyncNotify))
	{
		/* check prefs, reregister for notifications */
		/* WE HAVE NO GLOBALS !!! */
		tagPrefs prefs;

		ReadPrefs(&prefs);
		
//		if (prefs.bEnabled)
		{
			RegisterNotifications();
		}
	}
	else if (cmd == sysAppLaunchCmdNotify)
	{
		/* handle power off and power on, handle volume mounted (card insert) */
		/* WE HAVE NO GLOBALS !!! */
		SysNotifyParamType *snpt = (SysNotifyParamType *) cmdPBP;
		if (snpt->notifyType == sysNotifyLateWakeupEvent)
		{
			/* Wake up, display our form, catch all events */
HostTraceOutputTL(appErrorClass, "LateWakeUp");
			HandleKeyGuard();
		}
		else if (snpt->notifyType == sysNotifyVirtualCharHandlingEvent)
		{
			if ((((SysNotifyVirtualCharHandlingType *) snpt->notifyDetailsP)->keyDown.chr == hsChrKeyboardLock))// && (((SysNotifyVirtualCharHandlingType *) snpt->notifyDetailsP)->keyDown.modifiers & poweredOnKeyMask))
			{
				/* Wake up, display our form, catch all events */
HostTraceOutputTL(appErrorClass, "KeyGuard");
				if (HandleKeyGuard())
					snpt->handled = true;
			}
			else if (((SysNotifyVirtualCharHandlingType *) snpt->notifyDetailsP)->keyDown.chr == hsChrWakeup)
			{
				UInt32 value = 0;
				if ((FtrGet(MyAppID, 2, &value) == errNone) && value)
				{
					EvtEnqueueKey(hsChrKeyboardLock, 0, poweredOnKeyMask | commandKeyMask);
//					HandleKeyGuard();
				}
/*
				if (CheckForTreo())
				{
					UInt32 keyguard;
					HsAttrGet(hsAttrKeyboardLocked, 0, &keyguard);
					if (keyguard)
					{
						EvtEnqueueKey(hsChrKeyboardLock, 0, poweredOnKeyMask | commandKeyMask);
					}
				}
*/
			}
		}
		else if (snpt->notifyType == sysNotifyVolumeMountedEvent)
		{
			tagPrefs prefs;

			ReadPrefs(&prefs);

			if (prefs.bCatchCardInsert)
			{
				snpt->handled = 0xff;//|= vfsHandledUIAppSwitch;
			}
		}
		else if (snpt->notifyType == pmNotifyHighPriorityUITask)
		{
			UInt32 value = 0;
			// still displayed?
			if ((FtrGet(MyAppID, 1, &value) == errNone) && value)
			{
				FtrSet('NxVe', 'Hk', 1);
			}
		}
	}

	HostTraceClose();

	return 0;
}


#define COPY_16_PIXEL \
	tmpBits2[y1 + x + 0] = (tmpBits1[y2 + x + 0] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 1] = (tmpBits1[y2 + x + 1] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 2] = (tmpBits1[y2 + x + 2] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 3] = (tmpBits1[y2 + x + 3] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 4] = (tmpBits1[y2 + x + 4] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 5] = (tmpBits1[y2 + x + 5] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 6] = (tmpBits1[y2 + x + 6] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 7] = (tmpBits1[y2 + x + 7] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 8] = (tmpBits1[y2 + x + 8] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 9] = (tmpBits1[y2 + x + 9] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 10] = (tmpBits1[y2 + x + 10] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 11] = (tmpBits1[y2 + x + 11] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 12] = (tmpBits1[y2 + x + 12] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 13] = (tmpBits1[y2 + x + 13] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 14] = (tmpBits1[y2 + x + 14] >> 1) & 0x7bef; \
	tmpBits2[y1 + x + 15] = (tmpBits1[y2 + x + 15] >> 1) & 0x7bef; \
	x += 16;

static
void fastcopy(UInt16 *tmpBits1, Int32 scrWidth, Int32 scrHeight, UInt16 *tmpBits2, Int32 dstHeight)
{
	Int32 x, y;
	Int32 y1 = 0;
	Int32 y2 = (scrHeight - dstHeight) * (Int32) scrWidth;

											for (y=0; y<dstHeight; y++)
											{
												x = 0;
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												COPY_16_PIXEL
												y1 += scrWidth;
												y2 += scrWidth;
											}
}
